import { jsxs as _jsxs } from "react/jsx-runtime";
import { Fragment as _Fragment } from "react/jsx-runtime";
import { jsx as _jsx } from "react/jsx-runtime";
import _inheritsLoose from "@babel/runtime/helpers/esm/inheritsLoose";
import _extends from "@babel/runtime/helpers/esm/extends";
import { createRef, Component, lazy, Suspense } from 'react';
import PropTypes from 'prop-types';
import cx from 'classnames';
import { withIntersectionObserver, withOpenToggle } from '@s-ui/hoc';
import { BASE_CLASS, CLASS_ARROW, CLASS_INNER, CLASS_TARGET, COLORS, PREFIX_PLACEMENT, PLACEMENTS } from './config';
import AtomTooltipExtendedChildren from './AtomTooltipExtendedChildren';
var Tooltip = /*#__PURE__*/lazy(function () {
  return import('reactstrap/lib/Tooltip');
});

var createClasses = function createClasses(array, sufix) {
  if (sufix === void 0) {
    sufix = '';
  }

  return array.reduce(function (res, key) {
    var _extends2;

    return _extends({}, res, (_extends2 = {}, _extends2[key] = BASE_CLASS + "--" + key + sufix, _extends2));
  }, {});
};

var COLOR_CLASSES = createClasses(COLORS, 'Color');

var AtomTooltip = /*#__PURE__*/function (_Component) {
  _inheritsLoose(AtomTooltip, _Component);

  function AtomTooltip() {
    var _this;

    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    _this = _Component.call.apply(_Component, [this].concat(args)) || this;
    _this.preventNonTouchEvents = false;
    _this.hasTouchEnded = false;
    _this.touchTimer = null;
    _this.onClickTarget = null;
    _this.title = null;
    _this.refTooltip = /*#__PURE__*/createRef();
    _this.refTarget = /*#__PURE__*/createRef();

    _this.handleContextMenu = function (e) {
      e.preventDefault();
      e.stopPropagation();
      return false;
    };

    _this.handleDragElement = function (e) {
      _this.handleToggle(e);
    };

    _this.handleClickOutsideElement = function (e) {
      var _this$props = _this.props,
          isOpen = _this$props.isOpen,
          onToggle = _this$props.onToggle;
      var target = _this.refTarget.current;

      if (isOpen) {
        var tooltipDom = _this.refTooltip.current;
        var isOutside = tooltipDom && !tooltipDom.contains(e.target);
        var isNotTarget = target && !target.contains(e.target);
        if (isOutside && isNotTarget) onToggle(e, {
          isOpen: false
        });
      }
    };

    _this.touchTimerCallback = function (e) {
      var onToggle = _this.props.onToggle;
      if (!_this.hasTouchEnded) onToggle(e);
      _this.preventNonTouchEvents = false;
      _this.hasTouchEnded = false;
    };

    _this.handleTouchStart = function () {
      var longPressTime = _this.props.longPressTime;
      _this.preventNonTouchEvents = true;
      _this.hasTouchEnded = false;
      clearTimeout(_this.touchTimer);
      _this.touchTimer = setTimeout(_this.touchTimerCallback, longPressTime);
      return false;
    };

    _this.handleTouchEnd = function (e) {
      if (!_this.preventNonTouchEvents) _this.handleStopPropagation(e);
      _this.hasTouchEnded = true;
      clearTimeout(_this.touchTimerCallback);
    };

    _this.handleStopPropagation = function (e) {
      e.preventDefault();
      e.stopPropagation();
      return false;
    };

    _this.handleToggleOnNormalTarget = function (e) {
      var type = e.type;
      var onToggle = _this.props.onToggle;
      var isValidTrigger = ['click', 'focusin', 'mouseover', 'mouseout'].includes(type);
      if (type === 'touchstart') _this.hasTouchEnded = false;
      if (type === 'touchend') _this.hasTouchEnded = true;

      if (_this.hasTouchEnded && ['focusin', 'mouseover'].includes(type)) {
        _this.handleStopPropagation(e);
      }

      if (isValidTrigger) onToggle(e);
    };

    _this.handleToggleOnCallToActionTarget = function (e) {
      var type = e.type;
      var onToggle = _this.props.onToggle;
      if (type === 'touchstart') _this.handleTouchStart(e);
      if (type === 'touchend') _this.handleTouchEnd(e);
      if (!_this.preventNonTouchEvents) onToggle(e);
    };

    _this.handleToggle = function (e) {
      if (_this.onClickTarget) _this.handleToggleOnCallToActionTarget(e);else _this.handleToggleOnNormalTarget(e);
    };

    return _this;
  }

  var _proto = AtomTooltip.prototype;

  _proto.extendChildren = function extendChildren() {
    var _this2 = this;

    var children = this.props.children; // eslint-disable-line react/prop-types

    var ref = this.refTarget;
    var className = CLASS_TARGET;
    var onTouchEnd = this.handleToggle;
    return /*#__PURE__*/_jsx(AtomTooltipExtendedChildren, {
      ref: ref,
      className: className,
      onTouchEnd: onTouchEnd,
      handleDragElement: this.handleDragElement,
      disableTitle: this.disableTitle,
      restoreTitle: this.restoreTitle,
      handleClickOutsideElement: this.handleClickOutsideElement,
      setter: function setter(_ref) {
        var onClick = _ref.onClick,
            title = _ref.title;
        _this2.onClickTarget = onClick;
        _this2.title = title;
      },
      children: children
    });
  };

  _proto.componentDidMount = function componentDidMount() {
    var target = this.refTarget.current;
    target.oncontextmenu = this.handleContextMenu;
  };

  _proto.componentWillUnmount = function componentWillUnmount() {
    clearTimeout(this.touchTimerCallback);
  };

  _proto.disableTitle = function disableTitle(e) {
    this.dataset.title = this.title;
    this.title = '';
  };

  _proto.restoreTitle = function restoreTitle(e) {
    this.title = this.dataset.title;
  };

  _proto.render = function render() {
    var _this$props2 = this.props,
        autohide = _this$props2.autohide,
        color = _this$props2.color,
        HtmlContent = _this$props2.content,
        delay = _this$props2.delay,
        hideArrow = _this$props2.hideArrow,
        placement = _this$props2.placement; // eslint-disable-line react/prop-types

    var target = this.refTarget.current;
    var restrictedProps = {
      autohide: autohide,
      delay: delay,
      hideArrow: hideArrow,
      placement: placement,
      target: target
    };
    var _this$props3 = this.props,
        isVisible = _this$props3.isVisible,
        isOpen = _this$props3.isOpen; // eslint-disable-line react/prop-types

    if (!isVisible && isOpen) isOpen = false;
    var classNames = cx(BASE_CLASS, color && COLOR_CLASSES[color]);
    return /*#__PURE__*/_jsxs(_Fragment, {
      children: [this.extendChildren(), target && /*#__PURE__*/_jsx(Suspense, {
        fallback: null,
        children: /*#__PURE__*/_jsx(Tooltip, _extends({}, restrictedProps, {
          isOpen: isOpen,
          toggle: this.handleToggle // eslint-disable-line
          ,
          className: classNames,
          innerClassName: CLASS_INNER,
          arrowClassName: CLASS_ARROW,
          placementPrefix: PREFIX_PLACEMENT,
          innerRef: this.refTooltip,
          offset: "auto,4px",
          children: HtmlContent ? /*#__PURE__*/_jsx(HtmlContent, {}) : this.title
        }))
      })]
    });
  };

  return AtomTooltip;
}(Component);

AtomTooltip.displayName = 'AtomTooltip';
AtomTooltip.defaultProps = {
  innerRef: function innerRef() {},
  isVisible: true,
  longPressTime: 1000,
  onToggle: function onToggle() {}
};
var ExportedAtomTooltip = withIntersectionObserver(withOpenToggle(AtomTooltip));
ExportedAtomTooltip.COLORS = COLORS;
ExportedAtomTooltip.PLACEMENTS = PLACEMENTS;
ExportedAtomTooltip.displayName = AtomTooltip.displayName;
AtomTooltip.COLORS = COLORS;
AtomTooltip.PLACEMENTS = PLACEMENTS;
export default ExportedAtomTooltip;
export { AtomTooltip as AtomTooltipBase };
export { COLORS as atomTooltipColors };
export { PLACEMENTS as atomTooltipPlacements };